/* ARCADEmini simulator (unix, console)
 * version 0.8 date 2003-10-31
 * part of the ARCADEmini project - http[s]://arcademini.schuermans.info/
 * Copyright (C) 2003 sphaera <sphaera@geekness.de>
 *                and 1stein <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "am_dev.h"
#include "display.h"

int end = 0;

//signal handler to end the program
void end_signal( int sig_no )
{
  sig_no += 0; //keep compiler-happy
  end = 1;
}

//main program
int main( int param_cnt, char * * params )
{
  char * input_file, * log_file;
  int param_err, param_no;
  FILE * p_file, * p_log_file;
  st_am_state am_state;
  struct termios stdin_settings_orig, stdin_settings_new;
  unsigned short msec;
  int input_cnt;
  unsigned char input;

  //issue short info
  puts( "ARCADEmini simulator (unix, console)\n"
        "version 0.8 date 2003-10-31\n"
        "part of the ARCADEmini project - http[s]://arcademini.schuermans.info/\n"
        "Copyright (C) 2003 sphaera <sphaera@geekness.de>\n"
        "                and 1stein <1stein@schuermans.info>\n"
        "Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html\n"
      );

  //read parameters
  param_err = 0;
  input_file = NULL;
  log_file = NULL;
  for( param_no = 1; param_no < param_cnt; param_no++ )
  {
    //logfile filename
    if( strcmp( params[param_no], "-l" ) == 0 || strcmp( params[param_no], "--log" ) == 0 )
    {
      param_no++;
      if( log_file != NULL || param_no >= param_cnt ) //already specified or no filename following
        param_err = 1;
      else
        log_file = params[param_no];
    }
    //unknown option
    else if( params[param_no][0] == '-' )
      param_err = 1;
    //input filename
    else
    {
      if( input_file != NULL ) //already specified
        param_err = 1;
      else
        input_file = params[param_no];
    }
  } //for( param_no ...

  //check parameters
  if( input_file == NULL )
    param_err = 1;

  //parameter error
  if( param_err )
  {
    //print syntax explanation
    puts( "syntax:     am_sim [options] <input-file>\n"
          "\n"
          "options:    -l / --log <logfile>\n"
          "\n"
          "example:    am_sim -l test.log test.ambin\n"
          "\n"
        );
    //exit with error
    return -1;
  }

  //open input file
  p_file = fopen( input_file, "rb" );
  if( p_file == NULL )
  {
    printf( "could not open input-file \"%s\"\n", input_file );
    return -1;
  }

  //open logfile
  if( log_file == NULL )
    p_log_file = NULL;
  else
  {
    p_log_file = fopen( log_file, "wt" );
    if( p_log_file == NULL )
    {
      printf( "could not open logfile \"%s\"\n", log_file );
      return -1;
    }
  }

  //initialize state
  if( am_dev_init_state( &am_state, p_file, p_log_file ) != 0 )
  {
    printf( "error in ambin-file \"%s\"\n", input_file );
    return -1;
  }

  //install signal handler to end program
  signal( SIGHUP, end_signal );
  signal( SIGINT, end_signal );
  signal( SIGTERM, end_signal );

  //set stdin to unbuffered mode
  tcgetattr( 0, &stdin_settings_orig );
  memcpy( &stdin_settings_new, &stdin_settings_orig, sizeof( struct termios ) );
  stdin_settings_new.c_lflag &= ~ICANON & ~ECHO;
  stdin_settings_new.c_cc[VMIN] = 1;
  stdin_settings_new.c_cc[VTIME] = 0;
  tcsetattr( 0, TCSANOW, &stdin_settings_new );

  //main loop
  printf( "\x01B""c" );
  while( !end )
  {
    //execute a command
    msec = am_dev_exec( &am_state );
    //display new picture if it changed
    if( am_state.picture_changed )
    {
      display_picture( am_state.picture );
      am_state.picture_changed = 0;
    }
    //wait for delay to end
    if( msec > 0 )
      usleep( (unsigned long)msec * 1000 );
    //process input
    for( ioctl( 0, FIONREAD, &input_cnt ); !end && input_cnt > 0; input_cnt-- )
    {
      //get next byte of input
      if( read( 0, &input, 1 ) == 1 )
      {
        switch( input )
        {
          case '1': am_state.keys_pressed |= 0x01; am_state.keys_released |= 0x01; break;
          case '2': am_state.keys_pressed |= 0x02; am_state.keys_released |= 0x02; break;
          case '3': am_state.keys_pressed |= 0x04; am_state.keys_released |= 0x04; break;
          case '4': am_state.keys_pressed |= 0x08; am_state.keys_released |= 0x08; break;
          case '5': am_state.keys_pressed |= 0x10; am_state.keys_released |= 0x10; break;
          case '6': am_state.keys_pressed |= 0x20; am_state.keys_released |= 0x20; break;
          case '7': am_state.keys_pressed |= 0x40; am_state.keys_released |= 0x40; break;
          case '8': am_state.keys_pressed |= 0x80; am_state.keys_released |= 0x80; break;
          case 0x1B: end = 1; break;
        }
      }
    }
  } //while( !end )

  //restore stdin settings
  tcsetattr( 0, TCSANOW, &stdin_settings_orig );

  //close input-file
  fclose( p_file );

  //close logfile
  if( p_log_file != NULL )
    fclose( p_log_file );

  //end
  return 0;
}
